// Zoom Ontop Fix
// Copyright (C) 2017 Philippe Troin <phil@fifi.org>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// You need both xdotool and xwininfo installed

const Lang = imports.lang;
const Main = imports.ui.main;
const Me   = imports.misc.extensionUtils.getCurrentExtension();

const TIMER_RATE_MILLIS = 2500;

let logLevel = 1;

const ZoomOnTop = new Lang.Class({
    Name: "ZoomOnTop",

    _init: function() {
	this._onWindowCreatedHandlerId   = null;
	this._timerId                    = null;
	this._transientsCount            = 0;
	this._enabled                    = false;
    },

    _log: function(l, s) {
	if (l <= logLevel) {
	    log(Me.uuid+': '+s);
	}
    },

    _windowMatches: function(window) {
    },

    _scheduleCallback: function() {
	if (this._timerId === null) {
	    this._timerId = Main.Mainloop.timeout_add(TIMER_RATE_MILLIS, Lang.bind(this, this._timerCallback));
	    this._log(1, 'timer enabled');
	    this._timerCallback();
	}
    },

    _cancelCallback: function() {
	if (this._timerId !== null) {
	    Main.Mainloop.source_remove(this._timerId);
	    this._timerId = null;
	    this._log(1, 'timer disabled');
	}
    },

    _timerCallback: function() {
	this._log(2, 'timer tick');
	Main.Util.spawn(['sh', '-c',
		     "xwininfo -root -tree | egrep '^[[:space:]]*0x[a-f0-9]+[[:space:]]+\"(zoom_linux_float_video_window|as_toolbar)\":[[:space:]]' | sed -e 's!^[[:space:]]*!!' -e 's![[:space:]].*$!!' | sort | xargs -rn 1 xdotool windowraise"]);
	return true;
    },

    _onWindowCreated: function(display, window) {
	if (this._enabled) {
	    let wclass = window.get_wm_class();
	    let title = window.get_title();
	    if (wclass === 'zoom' && title !== 'zoom' && window.is_skip_taskbar()) {
		if (title !== 'zoom_linux_float_video_window' && title !== 'as_toolbar') {
		    this._log(0, 'ERROR: unexpected unmanaged window name "'+title+'"');
		}
		if (this._transientsCount !== 0) {
		    this._cancelCallback();
		}
		this._scheduleCallback();
		this._transientsCount = this._transientsCount+1;
		this._log(1, 'zoom transients='+this._transientsCount);
		window.connect('unmanaged', Lang.bind(this, this._onWindowDestroyed));
	    }
	}
    },

    _onWindowDestroyed: function() {
	if (this._enabled) {
	    this._transientsCount = this._transientsCount-1;
	    if (this._transientsCount <= 0) {
		this._cancelCallback();
	    }
	    this._log(1, 'zoom transients='+this._transientsCount);
	}
    },

    enable: function() {
	this.disable();
	this._onWindowCreatedHandlerId = global.display.connect('window-created', Lang.bind(this, this._onWindowCreated));
	this._enabled = true;
	this._log(1, 'enabled');
    },

    disable: function() {
	this._enabled = false;
	if (this._onWindowCreatedHandlerId !== null) {
	    global.display.disconnect(this._onWindowCreatedHandlerId);
	}
	this._onWindowCreatedHandlerId = null;

	this._cancelCallback();
	this._log(1, 'disabled');
    },
});

let _zoomOnTop;

function init() {
    _zoomOnTop = null;
}

function enable() {
    if (_zoomOnTop === null) {
	_zoomOnTop = new ZoomOnTop();
	_zoomOnTop.enable();
    }
}

function disable() {
    if (_zoomOnTop !== null) {
	_zoomOnTop.disable();
	_zoomOnTop = null;
    }
}
